"""
The mod:`pyswarms.utils.environments` module implements various
optimization environments to analyze optimizer performance or search
better parameters
"""

from .plot_environment import PlotEnvironment

__all__ = [
    "PlotEnvironment"
    ]
